import React, { useState, useCallback, useMemo } from 'react';
import Card from './components/Card';
import TierCard from './components/TierCard';
import HistoryList from './components/HistoryList';
import type { Upgrades } from './components/GubbleStore';
import GubbleStore from './components/GubbleStore';

// --- Scratch Card Game Types and State ---
export type Tier = {
  name: string;
  unlockPrice: number;
  buyPrice: number;
  gubblePointChance: number;
};

const TIERS: Tier[] = [
  { name: 'Start', unlockPrice: 0, buyPrice: 1, gubblePointChance: 0.0 },
  { name: 'Bronze', unlockPrice: 3, buyPrice: 2, gubblePointChance: 0.0 },
  { name: 'Silver', unlockPrice: 7, buyPrice: 5, gubblePointChance: 0.0 },
  { name: 'Gold', unlockPrice: 15, buyPrice: 10, gubblePointChance: 0.1 },
  { name: 'Platinum', unlockPrice: 30, buyPrice: 20, gubblePointChance: 0.1 },
  { name: 'Diamond', unlockPrice: 75, buyPrice: 50, gubblePointChance: 0.2 },
  { name: 'Legend', unlockPrice: 750, buyPrice: 500, gubblePointChance: 0.2 },
  { name: 'Ultra', unlockPrice: 10000, buyPrice: 8000, gubblePointChance: 0.3 },
  { name: 'Ultimate', unlockPrice: 100000, buyPrice: 80000, gubblePointChance: 0.3 },
  { name: 'Champion', unlockPrice: 5000000, buyPrice: 4000000, gubblePointChance: 0.4 },
  { name: 'King', unlockPrice: 100000000, buyPrice: 80000000, gubblePointChance: 0.5 },
  { name: 'Queen', unlockPrice: 1000000000, buyPrice: 800000000, gubblePointChance: 0.6 },
  { name: 'Emperor', unlockPrice: 10000000000, buyPrice: 8000000000, gubblePointChance: 0.7 },
  { name: 'God', unlockPrice: 100000000000, buyPrice: 80000000000, gubblePointChance: 0.8 },
  { name: 'Goddess', unlockPrice: 1000000000000, buyPrice: 800000000000, gubblePointChance: 0.9 },
];

const START_MONEY = 6;
const START_UNLOCKED: string[] = [];

function App() {
  // --- Game State ---
  const [money, setMoney] = useState(START_MONEY);
  const [unlockedTiers, setUnlockedTiers] = useState<string[]>(START_UNLOCKED);
  const [gubblePoints, setGubblePoints] = useState(0);
  const [card, setCard] = useState<{
    winningNumbers: number[];
    fields: { value: number; scratched: boolean; won: number | null }[];
    tier: Tier | null;
  } | null>(null);
  const [history, setHistory] = useState<{
    tier: string;
    winnings: number;
    winningNumbers: number[];
    fields: number[];
  }[]>([]);
  // --- Gubble Store State ---
  const [isStoreOpen, setIsStoreOpen] = useState(false);
  const [upgrades, setUpgrades] = useState<Upgrades>({ evenDouble: false, oddDouble: false, allTripple: false, gubbleDouble: false, addFourthNumber: false });
  const [upgradeBought, setUpgradeBought] = useState(false);

  // --- Tier Unlock Logic ---
  const handleUnlockTier = useCallback((tier: Tier) => {
    setMoney(m => {
      if (m < tier.unlockPrice) return m;
      setUnlockedTiers(prev => prev.includes(tier.name) ? prev : [...prev, tier.name]);
      return m - tier.unlockPrice;
    });
  }, []);

  // --- Buy Card Logic ---
  const handleBuyCard = useCallback((tier: Tier) => {
    setMoney(m => {
      if (m < tier.buyPrice) return m;
      if (!unlockedTiers.includes(tier.name)) return m;
      // Generate unique winning numbers
      const winningNumbers: number[] = [];
      const winningCount = upgrades.addFourthNumber ? 4 : 3;
      while (winningNumbers.length < winningCount) {
        const n = Math.floor(Math.random() * 9) + 1;
        if (!winningNumbers.includes(n)) winningNumbers.push(n);
      }
      // Generate 6 random fields (1-9)
      const fields = Array.from({ length: 6 }, () => ({ value: Math.floor(Math.random() * 9) + 1, scratched: false, won: null }));
      setCard({ winningNumbers, fields, tier });
      setHistory(h => [
        { tier: tier.name, winnings: 0, winningNumbers, fields: fields.map(f => f.value) },
        ...h.slice(0, 9)
      ]);
      return m - tier.buyPrice;
    });
  }, [unlockedTiers, upgrades]);

  // --- Gubble Store Logic ---
  const handleBuyUpgrade = useCallback((upgrade: keyof Upgrades, cost: number) => {
    setGubblePoints(gp => {
      if (gp < cost) return gp;
      setUpgrades(u => ({ ...u, [upgrade]: true }));
      setUpgradeBought(true);
      return gp - cost;
    });
  }, []);

  const handleStoreClose = useCallback(() => {
    setIsStoreOpen(false);
    if (upgradeBought) {
      setMoney(START_MONEY);
      setUnlockedTiers(START_UNLOCKED);
      setCard(null);
      setHistory([]);
      setUpgradeBought(false);
    }
  }, [upgradeBought]);

  // --- Update history winnings after all fields scratched ---
  const updateHistoryWinnings = useCallback(() => {
    if (!card) return;
    const allScratched = card.fields.every(f => f.scratched);
    if (allScratched) {
      setHistory(h => {
        const [latest, ...rest] = h;
        if (!latest || latest.winningNumbers !== card.winningNumbers) return h;
        return [
          { ...latest, winnings: card.fields.reduce((sum, f) => sum + (f.won || 0), 0) },
          ...rest
        ];
      });
    }
  }, [card]);

  React.useEffect(updateHistoryWinnings, [card]);

  // --- Memoized winnings for current card ---
  const currentWinnings = useMemo(() => card ? card.fields.reduce((sum, f) => sum + (f.won || 0), 0) : 0, [card]);

  // --- UI ---
  return (
    <div className="min-h-screen bg-gray-100 flex flex-col items-center justify-center">
      <h1 className="text-3xl font-bold text-blue-600 mb-2">Scratch Card Game</h1>
      <div className="mb-4 text-lg">Money: <span className="font-mono font-bold">${money}</span></div>
      <div className="mb-4 text-lg">Gubble Points: <span className="font-mono font-bold">{gubblePoints}</span></div>
      <button
        className="mb-4 px-4 py-2 bg-purple-600 hover:bg-purple-700 text-white font-bold rounded disabled:opacity-50"
        disabled={gubblePoints < 1}
        onClick={() => setIsStoreOpen(true)}
      >
        Open Gubble Point Store
      </button>
      <GubbleStore
        isOpen={isStoreOpen}
        onClose={handleStoreClose}
        gubblePoints={gubblePoints}
        upgrades={upgrades}
        onBuyUpgrade={handleBuyUpgrade}
      />
      <div className="flex flex-wrap gap-6 mb-8 justify-center">
        {TIERS.map((tier) => (
          <TierCard
            key={tier.name}
            tier={tier}
            unlocked={unlockedTiers.includes(tier.name)}
            money={money}
            onUnlock={handleUnlockTier}
            onBuy={handleBuyCard}
            gubblePointChance={tier.gubblePointChance}
          />
        ))}
      </div>
      {card && (
        <>
          <div className="mb-2 text-sm text-gray-600">
            <span className="font-semibold">Winnings this card: </span>
            ${currentWinnings}
          </div>
          <Card card={card} setCard={setCard} setMoney={setMoney} setGubblePoints={setGubblePoints} upgrades={upgrades} />
        </>
      )}
      <HistoryList history={history} />
    </div>
  );
}

export default App;
